@@ -1,6 +1,7 @@
 //! File logger.
 use crate::build::BuilderCommon;
 use crate::misc;
+use crate::permissions::restrict_file_permissions;
 #[cfg(feature = "slog-kvfilter")]
 use crate::types::KVFilterParameters;
 use crate::types::{Format, OverflowStrategy, Severity, SourceLocation, TimeZone};
@@ -136,6 +137,19 @@ impl FileLoggerBuilder {
         self.appender.rotate_compress = compress;
         self
     }
+
+    /// Sets whether the log files should have restricted permissions.
+    ///
+    /// If `true` is specified, new log files will be created with the `600` octal permission
+    /// on unix systems.
+    /// On Windows systems, new log files will have an ACL which just contains the SID of
+    /// the owner.
+    ///
+    /// The default value is `false`.
+    pub fn restrict_permissions(&mut self, restrict: bool) -> &mut Self {
+        self.appender.restrict_permissions = restrict;
+        self
+    }
 }
 
 impl Build for FileLoggerBuilder {
@@ -154,7 +168,10 @@ impl Build for FileLoggerBuilder {
             }
             #[cfg(feature = "json")]
             Format::Json => {
-                let drain = slog_json::Json::default(self.appender.clone());
+                let drain = slog_json::Json::new(self.appender.clone())
+                    .set_flush(true)
+                    .add_default_keys()
+                    .build();
                 self.common.build_with_drain(drain)
             }
         };
@@ -176,6 +193,7 @@ struct FileAppender {
     wait_compression: Option<mpsc::Receiver<io::Result<()>>>,
     next_reopen_check: Instant,
     reopen_check_interval: Duration,
+    restrict_permissions: bool,
 }
 
 impl Clone for FileAppender {
@@ -193,6 +211,7 @@ impl Clone for FileAppender {
             wait_compression: None,
             next_reopen_check: Instant::now(),
             reopen_check_interval: self.reopen_check_interval,
+            restrict_permissions: self.restrict_permissions,
         }
     }
 }
@@ -212,6 +231,7 @@ impl FileAppender {
             wait_compression: None,
             next_reopen_check: Instant::now(),
             reopen_check_interval: Duration::from_millis(1000),
+            restrict_permissions: false,
         }
     }
 
@@ -239,10 +259,15 @@ impl FileAppender {
             // If the old file was externally deleted and we attempt to open a new one before releasing the old
             // handle, we get a Permission denied on Windows. Release the handle.
             self.file = None;
-            let file = file_builder
+
+            let mut file = file_builder
                 .append(!self.truncate)
                 .write(true)
                 .open(&self.path)?;
+
+            if self.restrict_permissions {
+                file = restrict_file_permissions(&self.path, file)?;
+            }
             self.written_size = file.metadata()?.len();
             self.file = Some(BufWriter::new(file));
         }
@@ -276,6 +301,28 @@ impl FileAppender {
 
         let _ = self.file.take();
 
+        #[cfg(windows)]
+        {
+            if let Err(err) = self.rotate_old_files() {
+                const ERROR_SHARING_VIOLATION: i32 = 32;
+
+                // To avoid the problem reported by https://github.com/sile/sloggers/issues/43,
+                // we ignore the error if its code is 32.
+                if err.raw_os_error() != Some(ERROR_SHARING_VIOLATION) {
+                    return Err(err);
+                }
+            }
+        }
+        #[cfg(not(windows))]
+        self.rotate_old_files()?;
+
+        self.written_size = 0;
+        self.next_reopen_check = Instant::now();
+        self.reopen_if_needed()?;
+
+        Ok(())
+    }
+    fn rotate_old_files(&mut self) -> io::Result<()> {
         for i in (1..=self.rotate_keep).rev() {
             let from = self.rotated_path(i)?;
             let to = self.rotated_path(i + 1)?;
@@ -290,10 +337,12 @@ impl FileAppender {
                 if self.rotate_compress {
                     let (plain_path, temp_gz_path) = self.rotated_paths_for_compression()?;
                     let (tx, rx) = mpsc::channel();
+                    let restrict_perms = self.restrict_permissions;
 
                     fs::rename(&self.path, &plain_path)?;
                     thread::spawn(move || {
-                        let result = Self::compress(plain_path, temp_gz_path, rotated_path);
+                        let result =
+                            Self::compress(plain_path, temp_gz_path, rotated_path, restrict_perms);
                         let _ = tx.send(result);
                     });
 
@@ -311,10 +360,6 @@ impl FileAppender {
             fs::remove_file(delete_path)?;
         }
 
-        self.written_size = 0;
-        self.next_reopen_check = Instant::now();
-        self.reopen_if_needed()?;
-
         Ok(())
     }
     fn rotated_path(&self, i: usize) -> io::Result<PathBuf> {
@@ -349,11 +394,20 @@ impl FileAppender {
         ))
     }
     #[cfg(feature = "libflate")]
-    fn compress(input_path: PathBuf, temp_path: PathBuf, output_path: PathBuf) -> io::Result<()> {
+    fn compress(
+        input_path: PathBuf,
+        temp_path: PathBuf,
+        output_path: PathBuf,
+        restrict_perms: bool,
+    ) -> io::Result<()> {
         let mut input = File::open(&input_path)?;
-        let mut temp = GzipEncoder::new(File::create(&temp_path)?)?;
-        io::copy(&mut input, &mut temp)?;
-        temp.finish().into_result()?;
+        let mut temp = File::create(&temp_path)?;
+        if restrict_perms {
+            temp = restrict_file_permissions(&temp_path, temp)?;
+        }
+        let mut output = GzipEncoder::new(temp)?;
+        io::copy(&mut input, &mut output)?;
+        output.finish().into_result()?;
 
         fs::rename(temp_path, output_path)?;
         fs::remove_file(input_path)?;
@@ -464,6 +518,14 @@ pub struct FileLoggerConfig {
     /// The default value is `drop_and_report`.
     #[serde(default)]
     pub overflow_strategy: OverflowStrategy,
+
+    /// Whether to restrict the permissions of log files.
+    ///
+    /// For details, see the documentation of [`restict_permissions`].
+    ///
+    /// [`restrict_permissions`]: ./struct.FileLoggerBuilder.html#method.restrict_permissions
+    #[serde(default)]
+    pub restrict_permissions: bool,
 }
 
 impl FileLoggerConfig {
@@ -491,6 +553,7 @@ impl Config for FileLoggerConfig {
         builder.rotate_keep(self.rotate_keep);
         #[cfg(feature = "libflate")]
         builder.rotate_compress(self.rotate_compress);
+        builder.restrict_permissions(self.restrict_permissions);
         if self.truncate {
             builder.truncate();
         }
@@ -514,6 +577,7 @@ impl Default for FileLoggerConfig {
             rotate_keep: default_rotate_keep(),
             #[cfg(feature = "libflate")]
             rotate_compress: false,
+            restrict_permissions: false,
         }
     }
 }
@@ -527,9 +591,9 @@ fn path_template_to_path(
     let timestamp_string = match timezone {
         TimeZone::Local => {
             let local_timestamp = Local.from_utc_datetime(&date_time.naive_utc());
-            local_timestamp.format(&timestamp_template)
+            local_timestamp.format(timestamp_template)
         }
-        TimeZone::Utc => date_time.format(&timestamp_template),
+        TimeZone::Utc => date_time.format(timestamp_template),
     }
     .to_string();
     let path_string = path_template.replace("{timestamp}", &timestamp_string);
